//----------------------------------------------------------------------------------------
// Imaging protocol
//----------------------------------------------------------------------------------------

#ifndef _HEADLINE_
#include "CPHeadline.h"
#endif

#ifndef __ICONS__
#include <Icons.h>
#endif

#if GENERATING68K
#pragma segment Imaging
#endif

typedef enum boxTextOptions {
	constrainH = 1,
	constrainV = 2,
	constrainHV = 3
} boxTextOptions;

void GetTextBounds( StringPtr text, Point numer, Point denom, Rect* bounds );
void GetTextRect( StringPtr text, Rect* bounds );
void BoxText( StringPtr myPString, Rect *dst, boxTextOptions options );

void GetTextBounds( StringPtr text, Point numer, Point denom,
						  Rect* bounds )
{
	FontInfo info;
	GrafPtr		curPort;
	
	Fixed hScale = FixDiv(numer.h, denom.h);
	Fixed vScale = FixDiv(numer.v, denom.v);
	GetPort(&curPort);
	GetFontInfo( &info );
	SetRect( bounds, 0, FixMul( -info.ascent, vScale),
				FixMul( StringWidth( text ), hScale),
				FixMul(info.descent + info.leading, vScale) );
	OffsetRect( bounds, curPort->pnLoc.h, curPort->pnLoc.v );
}

void GetTextRect( StringPtr text, Rect* bounds )
{
	Point identity;
	
	SetPt(&identity, 1, 1);
	GetTextBounds( text, identity, identity, bounds );
}

void BoxText( StringPtr myPString, Rect *dst, boxTextOptions options )
{
	Point		numer, denom;
	short 	txSize;
	Rect		src;
	GrafPtr	curPort;
	
	GetPort(&curPort);
	if (!options || options & ~(constrainH | constrainV)) {
		DrawString( myPString );
		return;
	}
	txSize = curPort->txSize;
	TextSize( 100 );	/* Pick a big size so our source metrics are precise. */
	MoveTo(0, 0);
	GetTextRect( myPString, &src );
	switch (options) {
		case constrainH:
			numer.h = numer.v = dst->right - dst->left;
			denom.h = denom.v = src.right - src.left;
			break;
		case constrainV:
			numer.h = numer.v = dst->bottom - dst->top;
			denom.h = denom.v = src.bottom - src.top;
			break;
		case (constrainH | constrainV):
			SetPt(&numer, dst->right - dst->left, dst->bottom - dst->top);
			SetPt(&denom, src.right - src.left, src.bottom - src.top);
			break;
	}
	if (denom.h && denom.v) {
		short baseline = FixRound(FixMul(Long2Fix(-src.top),FixDiv(numer.v,denom.v)));
		MoveTo( dst->left, dst->top + baseline );
		if (curPort->grafProcs)
			CallQDTextProc(curPort->grafProcs->textProc, *myPString, myPString+1, numer, denom );
		else
			StdText( *myPString, myPString+1, numer, denom );
	}
	TextSize( txSize );
}

//----------------------------------------------------------------------------------------

void CPHeadline::Draw(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_Draw
	EnteringMethod("\pCPHeadline::Draw");
	ODUnused(invalidShape);

	CFocus	foc(ev, facet);

	ODTypeToken view = facet->GetFrame(ev)->GetViewType(ev);

	if (view == fSession->Tokenize(ev, kODViewAsLargeIcon))
	{
		this->PrivDrawAsLargeIcon(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsSmallIcon))
	{
		this->PrivDrawAsSmallIcon(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsThumbnail))
	{
		this->PrivDrawAsThumbnail(ev, facet, invalidShape);
		return;
	}
	
	if (view == fSession->Tokenize(ev, kODViewAsFrame))
	{
		ODFrame*	frame     = facet->GetFrame(ev);
		ODShape*	usedShape = frame->GetUsedShape(ev, kODNULL);
		RgnHandle	rgn       = usedShape->GetQDRegion(ev);
	
		usedShape->Release(ev);
	
		Rect		rct     = (*rgn)->rgnBBox;
		RgnHandle	oldClip = NULL;
		RgnHandle	newClip;
	
		if (frame->IsRoot(ev))
		{
			GrafPtr	curPort;
			::GetPort(&curPort);
			Rect r = curPort->portRect;
			r.left = r.right  - 15;
			r.top  = r.bottom - 15;
			::GetClip(oldClip = NewRgn());
			::ClipRect(&r);
			::DrawGrowIcon(curPort);
			::GetClip(newClip = ::NewRgn());
			::DiffRgn(oldClip, newClip, newClip);
			::SetClip(newClip);
			::DisposeRgn(newClip);
			::EraseRect(&(curPort->portRect));
		}
	
		// draw a gratuitous string, and capture the image for thumbnail displau
		fThumbnailPicture = ::OpenPicture(&rct);
		const short kTextSize = 18;
		::TextSize(kTextSize);
		short x = (rct.right - rct.left - ::StringWidth(fTextData)) / 2;
		short y = (rct.bottom - rct.top - kTextSize) / 2;
//		::MoveTo(x, y);
//		::DrawString(fTextData);
		Str255	partMaker = "\pby PartMaker";
		x = (rct.right - rct.left - ::StringWidth(partMaker)) / 2;
		y = (rct.bottom - rct.top + kTextSize) / 2;
//		::MoveTo(x, y);
//		::DrawString(partMaker);
		// JWM
		if (frame->IsRoot(ev))
		{
			rct.right -= 15;
			rct.bottom -= 15;
		}
		::TextFont(fFontNo);
		BoxText(fTextData, &rct, constrainHV);
		::ClosePicture();
		::DrawPicture(fThumbnailPicture, &rct);
	
		if (oldClip) {
			::SetClip(oldClip);
			::DisposeRgn(oldClip);
		}

		// JWM (KLUDGE to do this at draw time...)
		frame->SetDroppable(ev, kODTrue);
	}
}

//----------------------------------------------------------------------------------------

static pascal OSErr PrivDetachIconResource(ResType theType, Handle *theIcon, void* yourDataPtr)
{
	if (theIcon != kODNULL && *theIcon != kODNULL)
	{
		::DetachResource(*theIcon);
		return ::ResError();
	}
	else
		return noErr;
}

//----------------------------------------------------------------------------------------

void CPHeadline::PrivDrawAsLargeIcon(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect32;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight(ev) == kODFullHighlight && facet->GetWindow(ev)->IsActive(ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite(&iconSuite, kHelloIconSuite, svAllLargeData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc(PrivDetachIconResource);
			error = ::ForEachIconDo(iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor(actionProc);
		}
	}
	
	::SetRect(&rect32, 0, 0, kODLargeIconSize, kODLargeIconSize);
	::PlotIconSuite(&rect32, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight(ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame(ev)->GetUsedShape(ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion(ev);
		
		::PenPat(&ODQDGlobals.gray);
		::FrameRgn(usedRgn);
		::PenNormal();
		
		usedShape->Release(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPHeadline::PrivDrawAsSmallIcon(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	Rect				rect16;
	IconTransformType 	transformType = ttNone;
	Handle				iconSuite;
	
	// Check to see if the icon is selected and in an active window
	if ( facet->GetHighlight(ev) == kODFullHighlight && facet->GetWindow(ev)->IsActive(ev) )
		transformType = ttSelected;
	
	// Draw the icon.
	{
		CUsingLibraryResources res;
		OSErr error = ::GetIconSuite(&iconSuite, kHelloIconSuite, svAllSmallData);		
	
		if ( error == noErr )
		{
			// Create a universal proc pointer for our icon action proc.
			IconActionUPP actionProc = NewIconActionProc(PrivDetachIconResource);
			error = ::ForEachIconDo(iconSuite, svAllLargeData, actionProc, kODNULL);
			DisposeRoutineDescriptor(actionProc);
		}
	}
	
	::SetRect(&rect16, 0, 0, kODSmallIconSize, kODSmallIconSize);
	::PlotIconSuite(&rect16, atAbsoluteCenter, transformType, iconSuite);

	if ( facet->GetHighlight(ev) == kODDimHighlight )
	{
		ODShape* usedShape = facet->GetFrame(ev)->GetUsedShape(ev, kODNULL);
		RgnHandle usedRgn = usedShape->GetQDRegion(ev);
		
		::PenPat(&ODQDGlobals.gray);
		::FrameRgn(usedRgn);
		::PenNormal();
		
		usedShape->Release(ev);
	}
}

//----------------------------------------------------------------------------------------

void CPHeadline::PrivDrawAsThumbnail(Environment* ev, ODFacet* facet, ODShape* invalidShape)
{
	// The last image displayed as frame is used as a Thumbnail
	Rect rct;
	::SetRect(&rct, 0, 0, kODThumbnailSize, kODThumbnailSize);
	if (fThumbnailPicture)
		::DrawPicture(fThumbnailPicture, &rct);
}
//----------------------------------------------------------------------------------------

void CPHeadline::CanvasUpdated(Environment* ev, ODCanvas* canvas)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasUpdated
	EnteringMethod("\pCPHeadline::CanvasUpdated");
	ODUnused(canvas);
}

//----------------------------------------------------------------------------------------

void CPHeadline::HighlightChanged(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_HighlightChanged
	EnteringMethod("\pCPHeadline::HighlightChanged");
	ODUnused(facet);
}

//----------------------------------------------------------------------------------------

ODULong CPHeadline::GetPrintResolution(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GetPrintResolution
	EnteringMethod("\pCPHeadline::GetPrintResolution");
	ODUnused(frame);
	return 0;
}

//----------------------------------------------------------------------------------------

void CPHeadline::FacetAdded(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetAdded
	EnteringMethod("\pCPHeadline::FacetAdded");

	ODFrame* frame = facet->GetFrame(ev);
	frame->RequestFrameShape(ev, this->PrivGetFrameShape(ev, frame), kODNULL);
		// First, PrivGetFrameShape is called to calculate the frame
		// shape we want.  However, we might not get it, due to the
		// needs/constraints of the containing part.  Therefore, after
		// calculating what we want, we see if we can get it by calling
		// RequestFrameShape.
		// RequestFrameShape either returns us what we passed it, or
		// returns something else because it thinks it knows best.
		// Either way, we then proceed with the returned result.
		// Note that this means that we have to be prepared to deal
		// with a frame shape other than we desire.

	this->PrivAdjustFacets(ev, frame);
		// Given a new negotiated frame shape, adjust our facets based
		// on it.  That's all there is to it.

	if (frame->IsRoot(ev))
	{
		ODBoolean succeeded = kODFalse;
		succeeded = fSession->GetArbitrator(ev)->RequestFocusSet(ev, fFocusSet,frame);
		if (succeeded)
		{
			this->FocusAcquired(ev, fSelectionFocus, frame);
			this->FocusAcquired(ev, fMenuFocus, frame);
			this->FocusAcquired(ev, fKeyFocus, frame);
		}
	}
}

//----------------------------------------------------------------------------------------

void CPHeadline::FacetRemoved(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_FacetRemoved
	EnteringMethod("\pCPHeadline::FacetRemoved");
}

//----------------------------------------------------------------------------------------

void CPHeadline::CanvasChanged(Environment* ev, ODFacet* facet)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CanvasChanged
	EnteringMethod("\pCPHeadline::CanvasChanged");
	ODUnused(facet);
}

//----------------------------------------------------------------------------------------

void CPHeadline::GeometryChanged(Environment* ev, ODFacet* facet,
												   ODBoolean clipShapeChanged,
												   ODBoolean externalTransformChanged)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_GeometryChanged
	EnteringMethod("\pCPHeadline::GeometryChanged");
	ODUnused(facet);
	ODUnused(clipShapeChanged);
	ODUnused(externalTransformChanged);
}