//----------------------------------------------------------------------------------------
// From Linking protocol
//----------------------------------------------------------------------------------------

#include <Link.xh>									// JWM
#include <LinkSpec.xh>
#include <LinkSrc.xh>

#ifndef _HEADLINE_
#include "CPHeadline.h"
#endif

#include "DartmouthODUtils.h"						// JWM

// JWM
ODLinkSource* CPHeadline::CreateLink(Environment* ev, ODByteArray* data)

// Creates a new link object.
//
// If a link already exists to the content identified by the data
// and size arguments, this method returns the link object;
// otherwise, it creates a new link object, puts in the initial
// data and returns it to the caller.
//
// This method identifies the content to be linked (by resolving
// the object specifier saved in the data parameter, or by some
// other means). It then creates a link object to represent the
// content data. The part must maintain information about what
// portion of its contents have been linked to it, so that it may
// notify link clients when that data has been changed. The link
// created is returned to the caller. Before calling this method,
// you must ensure that the data identifies some portion of this
// part's contents. After calling this method successfully, this
// part maintains a link to the identified content.

{
	EnteringMethod("\pCPHeadline::CreateLink");
ODUnused(data);

	if (fSrcLink != kODNULL)
		return fSrcLink;
		
	// Call this part's draft to create both the link source object
	//   used by this part, and the link object returned from this
	//   method and used by destinations of the link.

	fSrcLink = fStorageUnit->GetDraft(ev)->CreateLinkSource(ev, fSelf);
	this->UpdateSourceLink(ev, 0);

#if 0
	ODLinkKey key;
	ODVolatile(fSrcLink);
	ODVolatile(key);
	TRY
		if (fSrcLink->Lock(ev, 0, &key))
		{
		TRY
			ODStorageUnit* linkContentSU = 
				fSrcLink->GetContentStorageUnit(ev, key);
			
			// First create and/or focus the prop/value to write to
			if (!linkContentSU->Exists(ev, kODPropContents, kHeadlineKind, 0))
				linkContentSU->AddProperty(ev, kODPropContents)->AddValue(ev, kODAppleTEXT);
			else
				linkContentSU->Focus(ev, kODPropContents, kODPosSame, kODAppleTEXT, 0, kODPosSame);
				
			// Now we write out the property.
			SetValueEOF(ev,fTextData[0],linkContentSU);
			StorageUnitSetValue(linkContentSU, ev, fTextData[0], &fTextData[1]);
	
			// A new change ID is used to update the link the first time.
			ODChangeID change = fSession->UniqueChangeID(ev);
			fSrcLink->ContentChanged(ev, change, key);
			fSrcLink->Unlock(ev, key);
		CATCH_ALL
			fSrcLink->Unlock(ev, key);
			RERAISE;
		ENDTRY
		}
	CATCH(kODErrBrokenLinkSource)
		DebugStr("\pPictPart: ODLinkSource::Lock() returned broken link error\n");
		RemoveSourceLink(ev);
	ENDTRY
#endif

	// Get the link object from the link source, increment its reference
	//   count, and return it.  All destinations in this document will 
	//   reference the same link object.

	//ODLink* link = fSrcLink->GetLink(ev);
	//link->IncrementRefCount(ev);
	
	// update all my facets
	PrivSetDirty(ev);
	PrivInvalAllDisplayFrames(ev);
	
	return fSrcLink;
}

//----------------------------------------------------------------------------------------
// CPHeadline::LinkUpdated
//----------------------------------------------------------------------------------------
void CPHeadline::LinkUpdated(Environment* ev, ODLink* updatedLink, ODChangeID id)
{
	EnteringMethod("\pCPHeadline::LinkUpdated");
	ODUnused(id);

	ODStorageUnit* 	su;
	ODLinkKey 		key;
	ODBoolean		gotSomething = kODFalse;
	char			theText[kHeadlineTextMax];
	
	if (updatedLink->Lock(ev, 0, &key))
	{
		su = updatedLink->GetContentStorageUnit(ev, key);

		TRY
		{
//			if (updatedLink->GetChangeID(ev) != fChangeID)
//			{
				su->Focus(ev, kODPropContents, kODPosUndefined,   
						kODAppleTEXT, 0, kODPosUndefined);
				ODByteArray	value;
				SetODByteArray(kHeadlineTextMax - 1, kHeadlineTextMax - 1, theText, &value);
				su->GetValue(ev, &value);
				theText[su->GetSize(ev)] = '\0';
				SetData(ev, theText, updatedLink->GetChangeID(ev));
				fDestLinkInfo.change = updatedLink->GetChangeID(ev);
				fDestLinkInfo.changeTime = updatedLink->GetChangeTime(ev);
//			}
//			else
//				// link circle alert! (we don't source links, so doesn't matter for now
//				SysBeep(2);
		}
		CATCH_ALL
		{
		}
		ENDTRY;
		
		updatedLink->Unlock(ev, key);
	}
}

//----------------------------------------------------------------------------------------
// CPHeadline::UpdateSourceLink
//----------------------------------------------------------------------------------------
void CPHeadline::UpdateSourceLink(Environment* ev, ODChangeID changeID)
{
	if (fSrcLink == kODNULL)
		return;

	ODLinkKey linkKey;
	ODLinkSource* linkSource = fSrcLink;
	
	ODVolatile(linkSource);
	ODVolatile(linkKey);

	// DebugStr("\pUpdateSourceLink" );
	TRY
		if (linkSource->Lock(ev, 0, &linkKey))
		{
			TRY
				if (!changeID)
					changeID = fSession->UniqueChangeID(ev);
	
				// Remove the current link from the link
				linkSource->Clear(ev, fSession->UniqueChangeID(ev), linkKey);
								
				ODStorageUnit* linkContentSU = 
					linkSource->GetContentStorageUnit(ev, linkKey);		
				// Begin a transaction if objects are cloned into the link
				ODDraftKey draftKey = 0;
				ODDraft *draft = fStorageUnit->GetDraft(ev);
				ODVolatile(draft);
				ODVolatile(draftKey);		
				TRY
					draftKey = draft->BeginClone(ev,
											linkContentSU->GetDraft(ev),
											kODCloneToLink);
	
					// First create and/or focus the prop/value to write to
					if (!linkContentSU->Exists(ev, kODPropContents, kHeadlineKind, 0))
						linkContentSU->AddProperty(ev, kODPropContents)->AddValue(ev, kHeadlineKind);
					else
						linkContentSU->Focus(ev, kODPropContents, kODPosSame, kHeadlineKind, 0, kODPosSame);
						
					// Now we write out the property.
					SetValueEOF(ev,fTextData[0],linkContentSU);
					StorageUnitSetValue(linkContentSU, ev, fTextData[0], &fTextData[1]);
			
		
					draft->EndClone(ev, draftKey);
		
				CATCH_ALL
		
					if (draftKey != 0 )
						draft->AbortClone(ev, draftKey);
					RERAISE;
		
				ENDTRY
		
				// Inform the link that it has changed, so destinations can
				//   update.  The changeID parameter should be the change
				//   causing the link to update.
		
				linkSource->ContentChanged(ev, changeID, linkKey);
				linkSource->Unlock(ev, linkKey);
		
			CATCH_ALL
		
				// Clear a partially-updated link
				linkSource->Clear(ev, fSession->UniqueChangeID(ev), linkKey);
				linkSource->Unlock(ev, linkKey);
		
			ENDTRY
		}
	CATCH(kODErrBrokenLinkSource)
		DebugStr("\pHeadline: ODLinkSource::Lock() returned broken link error\n");
		RemoveSourceLink(ev);
	ENDTRY
}

//----------------------------------------------------------------------------------------
// CPHeadline::RemoveSourceLink
//----------------------------------------------------------------------------------------
void CPHeadline::RemoveSourceLink( Environment* ev)
{
	if (fSrcLink != kODNULL)
	{
		ODLinkSource* linkSource = fSrcLink;
		fSrcLink = kODNULL;
		linkSource->SetSourcePart(ev, kODNULL);
		linkSource->Release(ev);
		
		this->PrivSetDirty( ev );
		this->PrivInvalAllDisplayFrames(ev);
	}
}

//----------------------------------------------------------------------------------------
// CPHeadline::RemoveDestLink
//----------------------------------------------------------------------------------------
void CPHeadline::RemoveDestLink( Environment* ev )
{
	if (fDestLink != kODNULL)
	{
		if (fDestLinkInfo.autoUpdate)
			fDestLink->UnregisterDependent(ev, fSelf);
		fDestLink->Release(ev);
		fDestLink = kODNULL;
	
		this->PrivSetDirty( ev );
		this->PrivInvalAllDisplayFrames(ev);
	}
}

#if 0 //JWM
//----------------------------------------------------------------------------------------

ODLinkSource* CPHeadline::CreateLink(Environment* ev, ODByteArray* data)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_CreateLink
	EnteringMethod("\pCPHeadline::CreateLink");
	ODUnused(data);
	return kODNULL;
}

//----------------------------------------------------------------------------------------

void CPHeadline::LinkUpdated(Environment* ev, ODLink* updatedLink, ODChangeID id)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_LinkUpdated
	EnteringMethod("\pCPHeadline::LinkUpdated");
	ODUnused(updatedLink);
	ODUnused(id);
}
#endif // !JWM

//----------------------------------------------------------------------------------------

void CPHeadline::RevealLink(Environment* ev, ODLinkSource* linkSource)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RevealLink
	EnteringMethod("\pCPHeadline::RevealLink");
	ODUnused(linkSource);
}

//----------------------------------------------------------------------------------------

void CPHeadline::LinkStatusChanged(Environment* ev, ODFrame* frame)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_LinkStatusChanged
	EnteringMethod("\pCPHeadline::LinkStatusChanged");
	ODUnused(frame);
}

//----------------------------------------------------------------------------------------

void CPHeadline::EmbeddedFrameChanged(Environment* ev, ODFrame* frame, ODChangeID change)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_EmbeddedFrameChanged
	EnteringMethod("\pCPHeadline::EmbeddedFrameChanged");
	ODUnused(change);
	ODUnused(frame);
}
