//----------------------------------------------------------------------------------------
// Undo protocol
//----------------------------------------------------------------------------------------

#include <Clipbd.xh>								// JWM
#include <Undo.xh>

#ifndef _HEADLINE_
#include "CPHeadline.h"
#endif

#include "DartmouthODUtils.h"						// JWM

// JWM
//----------------------------------------------------------------------------------------
void CPHeadline::UndoAction(Environment* ev, ODActionData* actionState)

// Tells the part to undo the action that is described by actionState.
//
// Called by the undo object.

{
	EnteringMethod("\pCPHeadline::UndoAction");

	DumbPartActionData	*action = (DumbPartActionData *) GetODByteArrayData(actionState);
	
	switch (action->fActionCode)
	{
	case kODCommandCut:
		/* should really restore previous clipboard... */

		/* restore previous text data */
		this->SetData(ev, action->fActionData, 0);
		break;
		
	case kODCommandCopy:
		/* should really restore previous clipboard... */
		break;
		
	case kODCommandPaste:
	case kODCommandClear:
	case kHeadlineCommandDrop:
		/* restore previous text data */
		this->SetData(ev, action->fActionData, 0);
		break;

	case kODCommandPasteAs:
		/* remove link */
		this->RemoveDestLink(ev);

		/* restore previous text data */
		this->SetData(ev, action->fActionData, 0);
		break;
		
	case kHeadlineCommandChangeFont:
		/* restore previous font */
		Str255	fontname;
		
		GetFontName(action->fPrevFont, fontname);
		this->DoFontChange(ev, fontname);
		break;
	} /* switch fActionCode */
}

//----------------------------------------------------------------------------------------

void CPHeadline::RedoAction(Environment* ev, ODActionData* actionState)

// Tells the part to Redo the action described by actionState.
//
// Called by the undo object.

{
	EnteringMethod("\pCPHeadline::RedoAction");

	DumbPartActionData	*action = (DumbPartActionData *) GetODByteArrayData(actionState);
	
	switch (action->fActionCode)
	{
	case kODCommandCut:
	case kODCommandCopy:
		this->DoCutOrCopy(ev, action->fFrame, action->fActionCode == kODCommandCut);
		break;
	
	case kODCommandPaste:
		this->DoPaste(ev, action->fFrame);
		break;

	case kODCommandPasteAs:
		this->DoPasteAs(ev, action->fFrame, &action->fPasteAsRslt, true);
		break;

	case kODCommandClear:
		this->DoClear(ev);
		break;
		
	case kHeadlineCommandChangeFont:
		/* change font again */
		this->DoFontChange(ev, (StringPtr) action->fActionData);
		break;

	case kHeadlineCommandDrop:
		/* re-do drop */
		this->SetData(ev, action->fActionData + strlen(action->fActionData) + 1, 0);		
	} /* switch fActionCode */
}

#if 0
//----------------------------------------------------------------------------------------

void CPHeadline::UndoAction(Environment* ev, ODActionData* actionState)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_UndoAction
	EnteringMethod("\pCPHeadline::UndoAction");
	ODUnused(actionState);
}

//----------------------------------------------------------------------------------------

void CPHeadline::RedoAction(Environment* ev, ODActionData* actionState)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_RedoAction
	EnteringMethod("\pCPHeadline::RedoAction");
	ODUnused(actionState);
}
#endif

//----------------------------------------------------------------------------------------

void CPHeadline::DisposeActionState(Environment* ev, ODActionData* actionState,
								ODDoneState doneState)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_DisposeActionState
	EnteringMethod("\pCPHeadline::DisposeActionState");
	ODUnused(actionState);
	ODUnused(doneState);
}

//----------------------------------------------------------------------------------------

void CPHeadline::WriteActionState(Environment* ev, ODActionData* actionState,
								ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_WriteActionState
	EnteringMethod("\pCPHeadline::WriteActionState");
	ODUnused(actionState);
	ODUnused(storageUnitView);
}

//----------------------------------------------------------------------------------------

ODActionData CPHeadline::ReadActionState(Environment* ev,
								ODStorageUnitView* storageUnitView)
{
	// CodeWarrior: Cmd-Click or Search:Find Definition  Explain_ReadActionState
	EnteringMethod("\pCPHeadline::ReadActionState");
	ODUnused(storageUnitView);

	ODActionData data;
	data._maximum = 0;
	data._length = 0;
	data._buffer = kODNULL;
	return data;
}
